'use strict';

// import {useState, useEffect} from 'react';

const ColorModeContext = React.createContext({ toggleColorMode: () => {} });

function App() {
    const prefersDarkMode = MaterialUI.useMediaQuery('(prefers-color-scheme: dark)');
    const preferredBrowserColorMode = prefersDarkMode ? 'dark' : 'light';
    const persistedColorMode = localStorage.getItem("colorMode")

    const [mode, setMode] = React.useState(persistedColorMode || preferredBrowserColorMode);
    const colorMode = React.useMemo(() => {
            return {
                toggleColorMode: () => {
                    return setMode((prevMode) => {
                        let newColorMode = prevMode === 'light' ? 'dark' : 'light';
                        localStorage.setItem("colorMode", newColorMode);
                        return newColorMode;
                    });
                },
            };
        }, [],
    );

    const theme = React.useMemo(() => {
        return MaterialUI.createTheme({
            palette: {
                mode: mode,
            },
        });
    }, [mode]);

    return (
        <div>
            <ColorModeContext.Provider value={colorMode}>
                <MaterialUI.ThemeProvider theme={theme}>
                    <MaterialUI.CssBaseline enableColorScheme/>

                    <RootHeader/>
                    <RootBody/>
                    <RootFooter/>

                </MaterialUI.ThemeProvider>
            </ColorModeContext.Provider>
        </div>
    );
}

let target = ReactDOM.createRoot(document.getElementById('react_container'));
target.render(
    <React.StrictMode>
        <App/>
    </React.StrictMode>
)
